#! /usr/bin/env python

import cssutils

from color import *

class CSSColorRule:
    def __init__(self, rule):
        self.selectors = [] 
        self.color = None
        self.background_color = None

        for prop in rule.style:
            if prop.propertyValue.item(0).type == cssutils.css.Value.COLOR_VALUE:
                if prop.name == 'background-color':
                    self.background_color = prop.propertyValue.item(0) 
                elif prop.name == 'color':
                    self.color = prop.propertyValue.item(0)
        
        for selector in rule.selectorList:
            self.selectors.append(selector)

    def result(self):
        """ Return the contrast between color and background_color, rounded 
        to 2 decimals """

        if self.color != None and self.background_color != None:
            return round(contrast(self.color, self.background_color), 2)
        else:
            return None



#! /usr/bin/env python

# Test suite for color parameters calculations (luminance and contrast)

import cssutils
import unittest

from colorcheck.color import *

LIGHTBLUE = cssutils.css.ColorValue("lightblue")
RED = cssutils.css.ColorValue("red")
GREENYELLOW = cssutils.css.ColorValue("greenyellow")
ORANGE = cssutils.css.ColorValue("orange")

class TestColor(unittest.TestCase):
    def test_luminance(self):
        self.assertAlmostEqual(luminance(LIGHTBLUE), 0.6370914)
        self.assertAlmostEqual(luminance(RED), 0.2126)
        self.assertAlmostEqual(luminance(GREENYELLOW), 0.8060947)
        self.assertAlmostEqual(luminance(ORANGE), 0.4817027)

    def test_contrast(self):
        self.assertAlmostEqual(contrast(LIGHTBLUE, RED), 2.6164943)
        self.assertAlmostEqual(contrast(LIGHTBLUE, GREENYELLOW), 1.2459692)
        self.assertAlmostEqual(contrast(LIGHTBLUE, ORANGE), 1.2922474)
        self.assertAlmostEqual(contrast(RED, GREENYELLOW), 3.2600713)
        self.assertAlmostEqual(contrast(RED, ORANGE), 2.0247626)
        self.assertAlmostEqual(contrast(GREENYELLOW, ORANGE), 1.6101005)

if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python

def luminance(color):
    """ Relative luminance of the color """
    unit_red = color.red / 255.0
    unit_green = color.green / 255.0
    unit_blue = color.blue / 255.0

    if unit_red <= 0.03928:
        lum_red = unit_red / 12.92
    else:
        lum_red = ((unit_red + 0.055) / 1.055) ** 2.4
    if unit_green <= 0.03928:
        lum_green = unit_green / 12.92
    else:
        lum_green = ((unit_green + 0.055) / 1.055) ** 2.4
    if unit_blue <= 0.03928:
        lum_blue = unit_blue / 12.92
    else:
        lum_blue = ((unit_blue + 0.055) / 1.055) ** 2.4
        
    return 0.2126*lum_red + 0.7152*lum_green + 0.0722*lum_blue

def contrast(bcolor, color):
    """ Contrast between two colors """
    if luminance(bcolor) > luminance(color):
        return (luminance(bcolor) + 0.05) / (luminance(color) + 0.05)
    else:
        return (luminance(color) + 0.05) / (luminance(bcolor) + 0.05)




#! /usr/bin/env python

import argparse
import sys
import cssutils

from csscolorrule import *

__version__ = "0.1.0"

if __name__ == "__main__":
    args_parser = argparse.ArgumentParser()
    args_parser.add_argument("cssfile", nargs=1)
    args_parser.add_argument("--version", action="store_true",\
        help="output version information and exit")
    args = args_parser.parse_args()

    if args.version:
        print("colorcheck %s" % __version__)
        sys.exit(0)

    parser = cssutils.CSSParser(raiseExceptions=True)
    sheet = parser.parseFile(sys.argv[1])

    for rule in sheet:
        if rule.type == rule.STYLE_RULE:
            csscolorrule = CSSColorRule(rule)
            result = csscolorrule.result()
            if result != None:
                print("%s : %.2f" % (rule.selectorText, result))



#! /usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='colorcheck',
    version='0.1.0',
    description='Check color contrasts in Web pages',
    url='https://redmine.jpages.eu/projects/colorcheck',
    author='Jérémy Pagès',
    author_email='contact@mail.jpages.eu',
    license='GPL-3.0-or-later',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
    packages=find_packages(exclude=('test')),
    python_requires='>=3'
)
